% Sign Restrictions Search
%
% -----------------------------------------------------------------------
% Explanation:
% ===========
%	
% -----------------------------------------------------------------------

function [ identifiedModels , nModelsFound, nModelsChecked ] = signrestrictionsearch(models, S, SN, Z, nVars, hasZeroRestrictions, nModelsFound, nModelsChecked, opt)

% initialize progress bar
totalruns=0; ii=0; jj=0; h = waitbar(0,sprintf('model draw: %d, transformation: %d, models found: %d',ii,jj,nModelsFound),'CreateCancelBtn','setappdata(gcbf,''Cancel'',1)');
% BUG: waitbar does not work with keepAllValid == 0

% These two loops search for models fulfilling the sign restrictions.
% The outer loop draws a model from the posterior or goes through all
% models in the posterior (depending on the option chosen).
% The inner loop draws orthogonal matrices to transform the model and
% checks the zero / sign restrictions.
for ii = 1:opt.nModelDraws

    if opt.drawFromPosterior == 1
        % If the option to draw a model from the posterior was selected
        % draw one of the models using the uniform distribution.
        modelPick = randi(size(models,2));
    else
        % If the option to loop through all models was selected simply
        % use the current position in the loop.
        modelPick = ii;
    end
    
    % Pick out the model according to modelPick.
	currentModel = models(modelPick);
    
    % Initialize the while-loop (see below).
    jj = 1;	
    while jj <= opt.nTransformationsPerDraw
            % Update the progress bar to reflect the current transformation
            % and the number of models already found.
            if opt.keepAllValid
                waitbar(totalruns/(opt.nModelDraws*opt.nTransformationsPerDraw),h,sprintf('model draw: %d, transformation: %d, models found: %d',ii,jj,nModelsFound));
                if getappdata(h, 'Cancel'); break; end
            else
                waitbar(ii/opt.nModelDraws,h,sprintf('model draw: %d, transformation: %d, models found: %d',ii,jj,nModelsFound));
                if getappdata(h, 'Cancel'); break; end
            end
            % Copy the model to be transformed to another instance. This is
            % necessary to go through different transformations of the same
            % model.
            transformedModel = currentModel;
            % Initialize the algorithm by drawing a random normal kxk 
            % matrix. 
            X = randn(nVars);
            % Check whether there are any zero restrictions.
            if hasZeroRestrictions              
                % Initialize the Q matrix.
                Q = zeros(nVars);
            else
                if opt.isNoTransform
                   Q = eye(nVars); 
                else                
                    % Compute the QR decomposition of the random matrix X
                    [Q,r]=qr(X);
                    for ll=1:nVars
                        if r(ll,ll)<0
                            Q(:,ll)=-Q(:,ll); % reflection matrix
                        end
                    end
                end
            end            
            
            % Go through the shocks to be identified (maximum would be
            % equal to the number of variables = k). isSignsFulfilled will
            % be turned 0 if a restriction is not fulfilled and the loop
            % ends.
			kk = 1; isSignsFulfilled = 1;
            while kk <= nVars && isSignsFulfilled == 1
                if hasZeroRestrictions                  
                    % Construct R. The max()-part ensures that in the first
                    % run Q(kk-1) is the zero vector (as defined in Arias et 
					% al.) taken from the initialization of Q.
                    if any(any(Z(:,:,kk)))
                        % this shock has zero restrictions.
                        % stack them above the Q-matrix to generate R.
                        ZZ = Z(any(Z(:,:,kk),2),:,kk);
                        R = [ ZZ*transformedModel.fA ; Q(:,1:max(1,kk-1))' ];
                    else
                        % this shock does not have zero restrictions.
                        % use only the Q-matrix for R.
                        R = Q(:,1:max(1,kk-1))';
                    end
                    % Find a matrix N whose columns form an orthonormal 
                    % basis for the null space of R. 
                    N = null(R);
                    if isempty(N)
                        error('Unable to find a matrix whose columns form an orthonormal basis for the null space of R.')
                    end
                    % basis for the null space of R.
                    % Find a column Q(:,kk) that is the projection of 
                    % X(:,kk) onto the null space of R normalized to be of 
                    % unit length (orthonormal). We then have 
                    % norm(Q:,kk) == 1 and R*Q(:,kk) == 0.
                    Q(:,kk) = N*N'*X(:,kk)/norm(N*N'*X(:,kk));
                    if ~opt.checkNegativeQ && Q(kk,kk)<0
                        Q(:,kk) = Q(:,kk)*(-1);
                    end
                end

				% Check whether the sign restrictions are fulfilled.
				if any(any(S(:,:,kk) ~= 0) == 1)
					if min(S(:,:,kk)*transformedModel.fA*Q(:,kk)-SN(:,:,kk)*ones(size(S(:,:,kk),2),1)) >= 0
						% sign restrictions are fulfilled --> keep going
					elseif opt.checkNegativeQ && max(S(:,:,kk)*transformedModel.fA*Q(:,kk)+SN(:,:,kk)*ones(size(S(:,:,kk),2),1)) <= 0
						% opposite sign restrictions are fulfilled --> 
                        % change the sign of the column. The property of
                        % orthogonality is not compromised by this.
						Q(:,kk) = -Q(:,kk);
                    else
                        % The sign restrictions are not fulfilled. Jump to
                        % the next transformation / model.
						isSignsFulfilled = 0;
						break
					end
				end
				if kk == nVars
					% All signs are fulfilled.
                    % Increase the count of models found.
					nModelsFound = nModelsFound + 1;
                    % Transform the decomposition of the VC matrix from 
                    % PP' = vc to PQQ'P' = vc.
					transformedModel.orthP = currentModel.orthP*Q;
                    % Calculate the impulse responeses based on the
                    % transformed decomposition.
                    if opt.isHistDecomp
                        [ transformedModel.irf , ~ , transformedModel.phiStruct ] = calculateirf(transformedModel,size(currentModel.res,1));
                    else
                        [ transformedModel.irf , ~ , transformedModel.phiStruct ] = calculateirf(transformedModel,opt.nImpulseHorizon);
                    end
                    % Delete the stacked impulse responses as they are only
                    % required within the SR algorithm.
                    transformedModel = rmfield(transformedModel,'fA');
                    % Put the model into the array of models fulfilling the
                    % sign restrictions.
					identifiedModels(nModelsFound) = transformedModel;
                end
                kk = kk + 1;
            end
     
            nModelsChecked = nModelsChecked + 1;

            if opt.keepAllValid == 0 && isSignsFulfilled == 1
				% Only keep the first transformation that fits the sign 
                % restrictions for this model.
                break
            end
            
            if nModelsFound >= opt.nMatches
                % Stop while-loop if the requested number of models has been found
                break
            end
        % Totalruns is only needed for the progress bar.
        jj = jj + 1; totalruns = totalruns + 1;
    end
    
    jj = 1;
    % Update the progress bar with the new model draw and the number of 
    % models already found.
    if opt.keepAllValid
        waitbar(totalruns/(opt.nModelDraws*opt.nTransformationsPerDraw),h,sprintf('model draw: %d, transformation: %d, models found: %d',ii,jj,nModelsFound));
        if getappdata(h, 'Cancel'); break; end
    else
        waitbar(ii/opt.nModelDraws,h,sprintf('model draw: %d, transformation: %d, models found: %d',ii,jj,nModelsFound));
        if getappdata(h, 'Cancel'); break; end
    end
    
    if nModelsFound >= opt.nMatches
        % Stop for-loop if the requested number of models has been found
        
        break
    end
    
end
delete(h);

if ~nModelsFound
    error('No model fitting the restictions was found.')
end

end