function [ output , ctmModel ] = generateOutput( y, identifiedModels, S, Z, nModelsFound, nVars, opt )

    % Stack the impulse responses into one matrix for further calculations.
    % This matrix has the following structure:
    % Columns 1:nVars^2 contain the contemporaneous impulse responses.
    % The next nVars^2 columns contain the impulse responses at horizon 1.
    % ...
    % Each row contains the impulse responses from one identified model.
    temp.irf = cat(2,[identifiedModels.irf]);
    temp.irfStack = zeros(nModelsFound,(nVars^2)*(opt.nImpulseHorizon+1));
    for ii = 1:(opt.nImpulseHorizon+1)
        for jj = 1:nVars^2
            temp.irfStack(:,(ii-1)*(nVars^2)+jj) = temp.irf(ii,jj:nVars^2:end)';
        end
    end

    % Calculate the pointwise median and the pointwise 68 percentile band
    % around the median.
    temp.irfMedian = median(temp.irfStack,1);
    temp.irf95 = prctile(temp.irfStack,95,1);
    temp.irf84 = prctile(temp.irfStack,84,1);
    temp.irf16 = prctile(temp.irfStack,16,1);
    temp.irf05 = prctile(temp.irfStack,5,1);

    % Generate selection matrix to multiply sqDistanceFromMedian with.
    selectCTM = eye((opt.nCTMHorizon+1)*(nVars^2));
    if opt.narrowCTMSearch == 1
        for ii = 1:nVars
            if ~(any(any(S(:,:,ii) ~= 0) == 1) || any(any(Z(:,:,ii))))
                selectCTM(:,ii:nVars:end) = 0;
            end
        end   
    end

    % For each model take the impulse responses up to the horizon specified
    % in the options and calculate the difference to the pointwise median.
    % Divide this difference by the pointwise standard deviation across
    % the models. Square the result (alternatively the absolute values could
    % be used), calculate sums over the models and select the model with 
    % the smallest sum.
    
    error = (temp.irfStack(:,1:(opt.nCTMHorizon+1)*(nVars^2)) - repmat(temp.irfMedian(:,1:(opt.nCTMHorizon+1)*(nVars^2)),nModelsFound,1))./repmat(std(temp.irfStack(:,1:(opt.nCTMHorizon+1)*nVars^2)),nModelsFound,1);

    % if 0 pointwise standard deviation are present (e.g. by
    % Cholesky) replace NaN with 0 after the division
    
    error(isnan(error)) = 0 ;
    
    sqDistanceFromMedian = power(error,2)*selectCTM;
    totalSqDistanceFromMedian = sum(sqDistanceFromMedian')';
    [~,closestImpulse] = min(totalSqDistanceFromMedian);
    ctmModel = identifiedModels(closestImpulse);
    output.irfCTM = ctmModel.irf;

    % Bring the calculated percentiles back to the regular form
    for ii = 1:nVars^2
        output.irfMedian(:,ii) = temp.irfMedian(ii:nVars^2:end)';
        output.irf95(:,ii) = temp.irf95(ii:nVars^2:end)';
        output.irf84(:,ii) = temp.irf84(ii:nVars^2:end)';
        output.irf16(:,ii) = temp.irf16(ii:nVars^2:end)';
        output.irf05(:,ii) = temp.irf05(ii:nVars^2:end)';
    end
    
    %% FEVD
    if opt.isFevdTable == 1    
        
        identifiedModels = calculateFEVD ( identifiedModels , nVars , opt );
        
        temp.fevd = cat(2,[identifiedModels.fevd]);
        temp.fevdStack = zeros(nModelsFound,(nVars^2)*(opt.nImpulseHorizon+1));
        for ii = 1:(opt.nImpulseHorizon+1)
            for jj = 1:nVars^2
                temp.fevdStack(:,(ii-1)*(nVars^2)+jj) = temp.fevd(ii,jj:nVars^2:end)';
            end
        end

        % Calculate the pointwise median and the pointwise 68 percentile band
        % around the median.
        temp.fevdMedian = median(temp.fevdStack,1);
        temp.fevd95 = prctile(temp.fevdStack,95,1);
        temp.fevd84 = prctile(temp.fevdStack,84,1);
        temp.fevd16 = prctile(temp.fevdStack,16,1);
        temp.fevd05 = prctile(temp.fevdStack,5,1);

        % Bring the calculated percentiles back to the regular form
        for ii = 1:nVars^2
            output.fevdMedian(:,ii) = temp.fevdMedian(ii:nVars^2:end)';
            output.fevd95(:,ii) = temp.fevd95(ii:nVars^2:end)';
            output.fevd84(:,ii) = temp.fevd84(ii:nVars^2:end)';
            output.fevd16(:,ii) = temp.fevd16(ii:nVars^2:end)';
            output.fevd05(:,ii) = temp.fevd05(ii:nVars^2:end)';
        end        
        
        
    end

    if opt.isStructShockPlots || opt.isStructShockTable
        % Structural shocks
        temp.structShock = cat(2,[identifiedModels.structShocks]);
        temp.structShockStack = zeros(nModelsFound,nVars*size(identifiedModels(1).res,1));
        for ii = 1:nVars
            temp.structShockStack(:,1+(ii-1)*size(temp.structShock,1):ii*size(temp.structShock,1)) = temp.structShock(:,ii:nVars:end)';
        end
        temp.structShockMedian = median(temp.structShockStack,1);
        temp.structShock95 = prctile(temp.structShockStack,95,1);
        temp.structShock84 = prctile(temp.structShockStack,84,1);
        temp.structShock16 = prctile(temp.structShockStack,16,1);
        temp.structShock05 = prctile(temp.structShockStack,5,1);   
        output.structShockMedian = zeros(size(identifiedModels(1).structShocks));
        output.structShock95 = output.structShockMedian;
        output.structShock84 = output.structShockMedian;
        output.structShock16 = output.structShockMedian;
        output.structShock05 = output.structShockMedian;
        for ii = 1:nVars
            output.structShockMedian(:,ii) = temp.structShockMedian(:,1+size(identifiedModels(1).res,1)*(ii-1):size(identifiedModels(1).res,1)*ii)';
            output.structShock95(:,ii) = temp.structShock95(:,1+size(identifiedModels(1).res,1)*(ii-1):size(identifiedModels(1).res,1)*ii)';
            output.structShock84(:,ii) = temp.structShock84(:,1+size(identifiedModels(1).res,1)*(ii-1):size(identifiedModels(1).res,1)*ii)';
            output.structShock16(:,ii) = temp.structShock16(:,1+size(identifiedModels(1).res,1)*(ii-1):size(identifiedModels(1).res,1)*ii)';
            output.structShock05(:,ii) = temp.structShock05(:,1+size(identifiedModels(1).res,1)*(ii-1):size(identifiedModels(1).res,1)*ii)';
        end
        
        x = lagmatrix(y,1:opt.nLags);
        temp2 = any(isnan(x),2);
        temp2(any(isnan(y),2)) = 1;
        temp2 = temp2(opt.nLags+1:end,:);
        if any(temp2)
            structShockMedian = NaN(length(temp2), nVars);
            structShock95 = structShockMedian;
            structShock84 = structShockMedian;
            structShock16 = structShockMedian;
            structShock05 = structShockMedian;
            temp3 = [];
            for ii = 1:length(temp2)
                if temp2(ii)==0
                    structShockMedian(ii,:)=output.structShockMedian(ii-length(temp3),:);
                    structShock95(ii,:)=output.structShock95(ii-length(temp3),:);
                    structShock84(ii,:)=output.structShock84(ii-length(temp3),:);
                    structShock16(ii,:)=output.structShock16(ii-length(temp3),:);
                    structShock05(ii,:)=output.structShock05(ii-length(temp3),:);
                else
                    temp3 = [temp3, 1];
                end
            end      
            output.structShockMedian = structShockMedian;
            output.structShock95 = structShock95;
            output.structShock84 = structShock84;
            output.structShock16 = structShock16;
            output.structShock05 = structShock05;
        end
    end
    
    if opt.isHistDecomp
        % Historical Decomposition
        temp.histDecomp = cat(2,[identifiedModels.HistDecomp]);
        temp.histDecompStack = zeros(nModelsFound,nVars*nVars*size(identifiedModels(1).res,1));
        for ii = 1:nVars*nVars
            temp.histDecompStack(:,1+(ii-1)*size(temp.histDecomp,1):ii*size(temp.histDecomp,1)) = temp.histDecomp(:,ii:nVars*nVars:end)';
        end
        temp.histDecompMedian = median(temp.histDecompStack,1);
        temp.histDecomp95 = prctile(temp.histDecompStack,95,1);
        temp.histDecomp84 = prctile(temp.histDecompStack,84,1);
        temp.histDecomp16 = prctile(temp.histDecompStack,16,1);
        temp.histDecomp05 = prctile(temp.histDecompStack,5,1);   
        output.histDecompMedian = zeros(size(identifiedModels(1).HistDecomp));
        output.histDecomp95 = output.histDecompMedian;
        output.histDecomp84 = output.histDecompMedian;
        output.histDecomp16 = output.histDecompMedian;
        output.histDecomp05 = output.histDecompMedian;
        for ii = 1:nVars*nVars
            output.histDecompMedian(:,ii) = temp.histDecompMedian(:,1+size(identifiedModels(1).res,1)*(ii-1):size(identifiedModels(1).res,1)*ii)';
            output.histDecomp95(:,ii) = temp.histDecomp95(:,1+size(identifiedModels(1).res,1)*(ii-1):size(identifiedModels(1).res,1)*ii)';
            output.histDecomp84(:,ii) = temp.histDecomp84(:,1+size(identifiedModels(1).res,1)*(ii-1):size(identifiedModels(1).res,1)*ii)';
            output.histDecomp16(:,ii) = temp.histDecomp16(:,1+size(identifiedModels(1).res,1)*(ii-1):size(identifiedModels(1).res,1)*ii)';
            output.histDecomp05(:,ii) = temp.histDecomp05(:,1+size(identifiedModels(1).res,1)*(ii-1):size(identifiedModels(1).res,1)*ii)';
        end
        
        x = lagmatrix(y,1:opt.nLags);
        temp2 = any(isnan(x),2);
        temp2(any(isnan(y),2)) = 1;
        temp2 = temp2(opt.nLags+1:end,:);
        if any(temp2)
            histDecompMedian = NaN(length(temp2), nVars*nVars);
            histDecomp95 = histDecompMedian;
            histDecomp84 = histDecompMedian;
            histDecomp16 = histDecompMedian;
            histDecomp05 = histDecompMedian;
            temp3 = [];
            for ii = 1:length(temp2)
                if temp2(ii)==0
                    histDecompMedian(ii,:)=output.histDecompMedian(ii-length(temp3),:);
                    histDecomp95(ii,:)=output.histDecomp95(ii-length(temp3),:);
                    histDecomp84(ii,:)=output.histDecomp84(ii-length(temp3),:);
                    histDecomp16(ii,:)=output.histDecomp16(ii-length(temp3),:);
                    histDecomp05(ii,:)=output.histDecomp05(ii-length(temp3),:);
                else
                    temp3 = [temp3, 1];
                end
            end      
            output.histDecompMedian = histDecompMedian;
            output.histDecomp95 = histDecomp95;
            output.histDecomp84 = histDecomp84;
            output.histDecomp16 = histDecomp16;
            output.histDecomp05 = histDecomp05;
        end
    end    
    
    % CTM - Forecast Error Variance Decomposition
    eyek = eye(nVars);
    output.ctmFevd = zeros(opt.nImpulseHorizon+1,nVars^2);
    for jj = 1:nVars
        for kk = 1:nVars
            tempSum = 0;
            tempSum2 = 0;
            for ii = 1:(opt.nImpulseHorizon+1)
                tempSum = tempSum + power(eyek(:,jj)'*ctmModel.phiStruct(:,(ii-1)*nVars+1:(ii-1)*nVars+nVars)*eyek(:,kk),2);
                tempSum2 = tempSum2 + ctmModel.phiStruct(:,(ii-1)*nVars+1:(ii-1)*nVars+nVars)*ctmModel.phiStruct(:,(ii-1)*nVars+1:(ii-1)*nVars+nVars)';
                output.ctmFevd(ii,(jj-1)*nVars+1+kk-1) = (tempSum/tempSum2(jj,jj))*100;
            end
        end
    end
    
end